@@ -1,26 +1,13 @@
 """Definition of Params"""
 import sys
-from typing import (
-    Optional,
-    Type,
-    Union,
-    List,
-    Callable,
-    Any,
-    Tuple,
-    Dict
-)
+from typing import Optional, Type, Union, List, Callable, Any, Tuple, Dict
 from pathlib import Path
-from diot import OrderedDiot
+from diot import OrderedDiot, Diot
 import rich
 from simpleconf import Config, LOADERS
 from .utils import (
-    always_list,
-    Namespace,
-    logger,
-    parse_type,
-    type_from_value,
-    parse_potential_argument
+    always_list, Namespace, logger, parse_type,
+    type_from_value, parse_potential_argument
 )
 from .defaults import POSITIONAL
 from .param import PARAM_MAPPINGS
@@ -28,7 +15,6 @@ from .help import HelpAssembler
 from .completer import Completer
 from .exceptions import PyParamNameError
 
-
 class Params(Completer): # pylint: disable=too-many-instance-attributes
     """Params, served as root params or subcommands
 
@@ -73,7 +59,6 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         command_groups: The ordered dict of command groups
         asssembler: The asssembler to assemble the help page
     """
-
     def __init__(self, # pylint: disable=too-many-arguments
                  names: Optional[Union[str, List[str]]] = None,
                  desc: Optional[Union[List[str], str]] = 'No description',
@@ -86,7 +71,6 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                  arbitrary: bool = False,
                  theme: str = 'default',
                  usage: Optional[Union[str, List[str]]] = None) -> None:
-        """Constructor"""
         self.desc: List[str] = always_list(desc, strip=False, split=False)
         self.prog: str = sys.argv[0] if prog is None else prog
         self.help_keys: List[str] = always_list(help_keys)
@@ -206,10 +190,13 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         Args:
             names: names of the argument or a parameter defined somewhere else
                 For example, in case we want to reuse a parameter
-                >>> param = cmd1.add_param('n,name')
-                >>> # reuse it:
-                >>> cmd2.add_param(param)
-                >>> # other arguments will be ignored, except force
+                ```python
+                param = cmd1.add_param('n,name')
+                # reuse it:
+                cmd2.add_param(param)
+                # other arguments will be ignored, except force
+                ```
+
             default: The default value for the argument
             type: The type of the argument
                 Including single value type and complex one
@@ -307,7 +294,7 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         return param
 
     def add_command(self, # pylint: disable=too-many-arguments
-                    names: Union[str, List[str]],
+                    names: Union["Params", str, List[str]],
                     desc: Optional[Union[str, List[str]]] = 'No description',
                     help_keys: Union[str, List[str]] = '__inherit__',
                     help_cmds: Union[str, List[str]] = '__inherit__',
@@ -323,6 +310,7 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
 
         Args:
             names: list of names of this command
+                It can also be a Params object that served as a subcommand
             desc: description of this command
             help_keys: help key for bring up help for this command
             help_cmds: help command for printing help for other
@@ -340,29 +328,37 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             The added command
         """
         # pylint: disable=too-many-locals
-        commands: List[str] = always_list(names)
-        command: "Params" = Params(
-            desc=desc,
-            prog=(f"{self.prog}{' [OPTIONS]' if self.params else ''} "
-                  f"{sorted(commands, key=len)[-1]}"),
-            help_keys=(self.help_keys if help_keys == '__inherit__'
-                       else help_keys),
-            help_cmds=(self.help_cmds if help_cmds == '__inherit__'
-                       else help_cmds),
-            help_on_void=(self.help_on_void if help_on_void == '__inherit__'
-                          else help_on_void),
-            help_callback=help_callback,
-            prefix=(self.prefix if prefix == '__inherit__'
-                    else prefix),
-            arbitrary=(self.arbitrary
-                       if arbitrary == '__inherit__'
-                       else arbitrary),
-            theme=(self.theme if theme == '__inherit__'
-                   else theme),
-            usage=usage,
-            names=commands
-        )
-        for cmd in commands:
+        command: Optional["Params"] = None
+        if isinstance(names, Params):
+            command = names
+            command.prog = (
+                f"{self.prog}{' [OPTIONS]' if self.params else ''} "
+                f"{command.name('long')}"
+            )
+        else:
+            names: List[str] = always_list(names)
+            command: "Params" = Params(
+                desc=desc,
+                prog=(f"{self.prog}{' [OPTIONS]' if self.params else ''} "
+                      f"{sorted(names, key=len)[-1]}"),
+                help_keys=(self.help_keys if help_keys == '__inherit__'
+                           else help_keys),
+                help_cmds=(self.help_cmds if help_cmds == '__inherit__'
+                           else help_cmds),
+                help_on_void=(self.help_on_void if help_on_void == '__inherit__'
+                              else help_on_void),
+                help_callback=help_callback,
+                prefix=(self.prefix if prefix == '__inherit__'
+                        else prefix),
+                arbitrary=(self.arbitrary
+                           if arbitrary == '__inherit__'
+                           else arbitrary),
+                theme=(self.theme if theme == '__inherit__'
+                       else theme),
+                usage=usage,
+                names=names
+            )
+        for cmd in command.names:
             # check if command has been added
             if not force and (cmd in self.params or cmd in self.commands):
                 raise PyParamNameError(
@@ -432,11 +428,14 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         return namespace
 
     def parse(self,
-              args: Optional[List[str]] = None) -> Namespace:
+              args: Optional[List[str]] = None,
+              help_modifier: Optional[Callable] = None) -> Namespace:
         """Parse the arguments from the command line
 
         Args:
             args: The arguments to parse
+            help_modifier: A callback to modify the help parameter and command.
+                It takes 2 elements, help parameter and command if any.
 
         Return:
             Namespace: The namespace of parsed arguments
@@ -448,8 +447,9 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                        desc='Print help information for this command',
                        force=True)
 
+        help_cmd: Optional["Params"] = None
         if self.commands:
-            help_cmd: "Params" = self.add_command(
+            help_cmd = self.add_command(
                 self.help_cmds,
                 desc='Print help of sub-commands',
                 force=True
@@ -461,6 +461,9 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                 desc="Command name to print help for"
             )
 
+        if callable(help_modifier):
+            help_modifier(self.get_param(self.help_keys[0]), help_cmd)
+
         if args is None:
             # enable completion only when we are trying to parse sys.argv
             if self.comp_shell:
@@ -511,7 +514,6 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         prev_param: Optional[Type['Param']] = None
         for i, arg in enumerate(args):
             logger.debug("- Parsing item %r", arg)
-
             # Match the arg with defined parameters
             # If arbitrary, non-existing parameters will be created on the fly
             # This means
@@ -558,7 +560,6 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
                 else:
                     logger.debug("  Param %r consumes %r",
                                  prev_param.namestr(), param_value)
-
             else: # neither
                 # Type: Optional[Type['Param']], Optional[str]
                 prev_param, matched = self._match_command_or_positional(
@@ -644,9 +645,7 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
 
     def _match_param(self, arg: str) -> Tuple[
             Optional[Type['Param']],
-            Optional[str],
-            Optional[str],
-            Optional[str]
+            Optional[str], Optional[str], Optional[str]
     ]:
         """Check if arg matches any predefined parameters. With
         arbitrary = True, parameters will be defined on the fly.
@@ -732,13 +731,109 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         param.hit = True
         if param_value is not None:
             param.push(param_value)
-
         return param, param_name, param_type, param_value
 
+    def to_dict(self) -> Diot:
+        """Save the parameters/commands to file.
+
+        This is helpful if the parameters/commands take time to load. Once can
+        cache this to a file, and load it from it using `from_file`.
+
+        Returns:
+            The complied Diot of parameters and commands
+        """
+        # load all parameters and commands
+        loaded: Diot = Diot(params={}, commands={})
+        param_groups = {}
+        for group, param_list in self.param_groups.items():
+            for param in param_list:
+                param_groups[param.names[0]] = group
+
+        for param in self._all_params():
+            param_name: str = ("POSITIONAL" if param.names[0] == POSITIONAL
+                               else param.names[0])
+            loaded.params[param_name] = Diot()
+            param_dict: Diot = loaded.params[param_name]
+            if len(param.names) > 1:
+                param_dict.aliases = ["POSITIONAL" if name == POSITIONAL
+                                      else name for name in param.names[1:]]
+            param_dict.default = param.default
+            param_dict.type = param.typestr()
+            param_dict.desc = param.desc
+            param_dict.show = param.show
+            param_dict.required = param.required
+            param_dict.type_frozen = param.type_frozen
+            param_dict.argname_shorten = param.argname_shorten
+            param_dict.group = group = param_groups[param.names[0]]
+
+        command_groups = {}
+        for group, command_list in self.command_groups.items():
+            for command in command_list:
+                for name in command.names:
+                    command_groups[name] = group
+
+        for command_name, command in self.commands.items():
+            if any(name in loaded.commands for name in command.names):
+                continue
+            loaded.commands[command_name] = Diot()
+            cmd_dict: Diot = loaded.commands[command_name]
+            if len(command.names) > 1:
+                cmd_dict.aliases = [name for name in command.names
+                                    if name != command_name]
+            cmd_dict.desc = command.desc
+            cmd_dict.help_keys = command.help_keys
+            cmd_dict.help_cmds = command.help_cmds
+            cmd_dict.help_on_void = command.help_on_void
+            cmd_dict.prefix = command.prefix
+            cmd_dict.arbitrary = command.arbitrary
+            cmd_dict.theme = command.theme
+            cmd_dict.usage = command.usage
+            cmd_dict.group = command_groups[command_name]
+            cmd_dict |= command.to_dict()
+
+        return loaded
+
+    def to_file(self,
+                path: Union[str, Path],
+                cfgtype: Optional[str] = None) -> None:
+        """Save the parameters/commands to file.
+
+        This is helpful if the parameters/commands take time to load. Once can
+        cache this to a file, and load it from it using `from_file`.
+
+        Args:
+            path: The path to the file
+            cfgtype: The type of the file
+                If not given, will inferred from the suffix of the path
+                Supports one of yaml, toml and json
+        """
+        loaded: Diot = self.to_dict()
+        if not isinstance(path, Path):
+            path = Path(path)
+        if not cfgtype:
+            if path.suffix in ('.yml', '.yaml'):
+                cfgtype = 'yaml'
+            elif path.suffix == '.toml':
+                cfgtype = 'toml'
+            elif path.suffix == '.json':
+                cfgtype = 'json'
+            else:
+                cfgtype = path.suffix
+        if cfgtype not in ('yaml', 'json', 'toml'):
+            raise ValueError('File type not supported: %s' % cfgtype)
+
+        if cfgtype == 'yaml':
+            loaded.to_yaml(path)
+        elif cfgtype == 'json':
+            loaded.to_json(path)
+        else:
+            loaded.to_toml(path)
+
     def from_file(self,
                   filename: Union[str, dict],
                   filetype: Optional[str] = None,
-                  show: bool = True) -> None:
+                  show: bool = True,
+                  force: bool = False) -> None:
         """Load parameters from file
 
         We support 2 types for format to load the parameters.
@@ -774,19 +869,19 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
         Args:
             filename: path to the file
             filetype: The type of the file. If None, will infer from the
-                filename.
-                Supported types: ini, cfg, conf, config, yml, yaml, json
-                env, osenv, toml
+                filename. Supported types: ini, cfg, conf, config, yml, yaml,
+                json, env, osenv, toml
             show: The default show value for parameters in the file
+            force: Whether to force adding params/commands
         """
         config: Config = Config(with_profile=False)
         config._load(filename, factory=LOADERS.get(filetype))
-
-        self.from_dict(config, show=show)
+        self.from_dict(config, show=show, force=force)
 
     def _from_dict_with_sections(self,
                                  dict_obj: Dict[str, Dict[str, dict]],
-                                 show: bool = True) -> None:
+                                 show: bool = True,
+                                 force: bool = False) -> None:
         """Load from the dict with 'params' and/or 'commands' sections"""
 
         param_section: Dict[str, dict] = dict_obj.get('params', {})
@@ -795,23 +890,29 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             param_attrs.setdefault('show', show)
             names: List[str] = always_list(param_attrs.pop('aliases', []))
             names.insert(0, param_name)
-            self.add_param(names, **param_attrs)
+            self.add_param([POSITIONAL if name == 'POSITIONAL'
+                            else name for name in names],
+                           **param_attrs, force=force)
 
         command_section: Dict[str, dict] = dict_obj.get('commands', {})
         for command_name, command_attrs in command_section.items():
             names: List[str] = always_list(command_attrs.pop('aliases', []))
             names.insert(0, command_name)
             param_section: Dict[str, dict] = command_attrs.pop('params', {})
-            command: "Params" = self.add_command(names, **command_attrs)
-            command.from_dict({"params": param_section}, show)
+            subcmd_section: Dict[str, dict] = command_attrs.pop('commands', {})
+            command: "Params" = self.add_command(names, **command_attrs,
+                                                 force=force)
+            command.from_dict({"params": param_section,
+                               "commands": subcmd_section}, show)
 
-    def from_dict(self, dict_obj: dict, show: bool = True):
+    def from_dict(self, dict_obj: dict, show: bool = True, force: bool = False):
         """Load parameters from python dict
 
         Args:
             dict_obj: A python dictionary to load parameters from
             show: The default show value for the parameters in the
                 dictionary
+            force: Whether to force adding params/commands
         """
         if 'params' not in dict_obj and 'commands' not in dict_obj:
             # express way
@@ -842,13 +943,15 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
 
             dict_obj = {"params": all_params}
 
-        self._from_dict_with_sections(dict_obj, show)
+        self._from_dict_with_sections(dict_obj, show, force)
 
     def from_arg(self, # pylint: disable=too-many-arguments
                  names: Union[str, List[str], 'ParamPath'],
                  desc: Union[str, List[str]] = "The configuration file.",
                  group: Optional[str] = None,
                  default: Optional[Union[str, Path]] = None,
+                 show: bool = True,
+                 force: bool = False,
                  args: Optional[List[str]] = None):
         """Load parameters from the file specified by naargument
         from command line
@@ -864,6 +967,8 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             desc: The description of the parameter
             group: The group of the parameter
             default: The default value of the file path
+            show: Whether those parameters should show up in the help page
+            force: Whether to force adding the parameters/commands
             args: The list of items to parse, otherwise
                 parse sys.argv[1:]
         """
@@ -877,7 +982,6 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
 
         prefixed_names: List[str] = [param._prefix_name(name)
                                      for name in param.names]
-
         args: List[str] = sys.argv[1:] if args is None else args
         args_with_the_arg: List[str] = []
         for i, arg in enumerate(args):
@@ -891,6 +995,5 @@ class Params(Completer): # pylint: disable=too-many-instance-attributes
             if len(args_with_the_arg) > 1
             else param.value
         )
-
         if filepath:
-            self.from_file(filepath)
+            self.from_file(filepath, show=show, force=force)
